@@ -11,7 +11,7 @@ public class ClassTransformerChain extends AbstractClassTransformer {
     private ClassTransformer[] chain;
     
     public ClassTransformerChain(ClassTransformer[] chain) {
-        this.chain = (ClassTransformer[])chain.clone();
+        this.chain = chain.clone();
     }
 
     public void setTarget(ClassVisitor v) {
@@ -32,7 +32,7 @@ public class ClassTransformerChain extends AbstractClassTransformer {
     }
 
     public String toString() {
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         sb.append("ClassTransformerChain{");
         for (int i = 0; i < chain.length; i++) {
             if (i > 0) {
