@@ -26,10 +26,12 @@ import javassist.CtMethod;
 import javassist.CtNewConstructor;
 import javassist.Modifier;
 import javassist.NotFoundException;
+import javassist.bytecode.AccessFlag;
 import javassist.bytecode.AttributeInfo;
 import javassist.bytecode.ClassFile;
 import javassist.bytecode.CodeAttribute;
 import javassist.bytecode.DuplicateMemberException;
+import javassist.bytecode.FieldInfo;
 import javassist.bytecode.InnerClassesAttribute;
 import javassist.expr.ConstructorCall;
 import javassist.expr.ExprEditor;
@@ -45,9 +47,6 @@ import static org.powermock.core.transformers.TransformStrategy.CLASSLOADER;
 import static org.powermock.core.transformers.TransformStrategy.INST_REDEFINE;
 import static org.powermock.core.transformers.TransformStrategy.INST_TRANSFORM;
 
-/**
- *
- */
 public abstract class AbstractMainMockTransformer implements MockTransformer {
 
     private static final String VOID = "";
@@ -56,6 +55,15 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
 
     public AbstractMainMockTransformer(TransformStrategy strategy) {this.strategy = strategy;}
 
+    public CtClass transform(final CtClass clazz) throws Exception {
+        if (clazz.isFrozen()) {
+            clazz.defrost();
+        }
+        return transformMockClass(clazz);
+    }
+
+    protected abstract CtClass transformMockClass(CtClass clazz) throws CannotCompileException, NotFoundException;
+
     protected String allowMockingOfPackagePrivateClasses(final CtClass clazz) {
         final String name = clazz.getName();
         if (strategy != INST_REDEFINE) {
@@ -73,7 +81,8 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
         return name;
     }
 
-    protected void suppressStaticInitializerIfRequested(final CtClass clazz, final String name) throws CannotCompileException {
+    protected void suppressStaticInitializerIfRequested(final CtClass clazz,
+                                                        final String name) throws CannotCompileException {
         if (strategy == CLASSLOADER) {
             if (MockGateway.staticConstructorCall(name) != MockGateway.PROCEED) {
                 CtConstructor classInitializer = clazz.makeClassInitializer();
@@ -141,7 +150,7 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
      * When that limit is exceeded class loader will fail to load the class.
      * Since instrumentation can increase method size significantly it must be
      * ensured that JVM limit is not exceeded.
-     * <p>
+     * <p/>
      * When the limit is exceeded method's body is replaced by exception throw.
      * Method is then instrumented again to allow mocking and suppression.
      *
@@ -151,10 +160,10 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
         for (CtMethod method : clazz.getDeclaredMethods()) {
             if (isMethodSizeExceeded(method)) {
                 String code = "{throw new IllegalAccessException(\"" +
-                        "Method was too large and after instrumentation exceeded JVM limit. " +
-                        "PowerMock modified the method to allow JVM to load the class. " +
-                        "You can use PowerMock API to suppress or mock this method behaviour." +
-                        "\");}";
+                                      "Method was too large and after instrumentation exceeded JVM limit. " +
+                                      "PowerMock modified the method to allow JVM to load the class. " +
+                                      "You can use PowerMock API to suppress or mock this method behaviour." +
+                                      "\");}";
                 method.setBody(code);
                 modifyMethod(method);
             }
@@ -168,46 +177,89 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
     }
 
     private void modifyMethod(final CtMethod method) throws NotFoundException, CannotCompileException {
-        if (!Modifier.isAbstract(method.getModifiers())) {
+
+        if (!shouldSkipMethod(method)) {
+
             // Lookup the method return type
+
             final CtClass returnTypeAsCtClass = method.getReturnType();
             final String returnTypeAsString = getReturnTypeAsString(method);
 
             if (Modifier.isNative(method.getModifiers())) {
-                String methodName = method.getName();
-                String returnValue = "($r)value";
-
-                if (returnTypeAsCtClass.equals(CtClass.voidType)) {
-                    returnValue = VOID;
-                }
-
-                String classOrInstance = "this";
-                if (Modifier.isStatic(method.getModifiers())) {
-                    classOrInstance = "$class";
-                }
-                method.setModifiers(method.getModifiers() - Modifier.NATIVE);
-                String code = "Object value = " + MockGateway.class.getName() + ".methodCall(" + classOrInstance + ", \"" + method.getName()
-                        + "\", $args, $sig, \"" + returnTypeAsString + "\");" + "if (value != " + MockGateway.class.getName() + ".PROCEED) "
-                        + "return " + returnValue + "; " + "throw new java.lang.UnsupportedOperationException(\"" + methodName + " is native\");";
-                method.setBody("{" + code + "}");
-                return;
+                modifyNativeMethod(method, returnTypeAsCtClass, returnTypeAsString);
+            } else {
+                modifyMethod(method, returnTypeAsCtClass, returnTypeAsString);
             }
-
-            final String returnValue = getCorrectReturnValueType(returnTypeAsCtClass);
-
-            String classOrInstance = "this";
-            if (Modifier.isStatic(method.getModifiers())) {
-                classOrInstance = "$class";
-            }
-
-            String code = "Object value = " + MockGateway.class.getName() + ".methodCall(" + classOrInstance + ", \"" + method.getName()
-                    + "\", $args, $sig, \"" + returnTypeAsString + "\");" + "if (value != " + MockGateway.class.getName() + ".PROCEED) " + "return "
-                    + returnValue + "; ";
-
-            method.insertBefore("{ " + code + "}");
         }
     }
 
+    private boolean shouldSkipMethod(CtMethod method) {
+        return isAccessFlagSynthetic(method) || Modifier.isAbstract(method.getModifiers());
+    }
+
+    private boolean isAccessFlagSynthetic(CtMethod method) {
+        int accessFlags = method.getMethodInfo2().getAccessFlags();
+        return ((accessFlags & AccessFlag.SYNTHETIC) != 0) && !isBridgeMethod(method);
+    }
+
+    private void modifyMethod(CtMethod method, CtClass returnTypeAsCtClass,
+                              String returnTypeAsString) throws CannotCompileException {
+        final String returnValue = getCorrectReturnValueType(returnTypeAsCtClass);
+
+        String classOrInstance = classOrInstance(method);
+
+        String code = "Object value = "
+                              + MockGateway.class.getName()
+                              + ".methodCall("
+                                + classOrInstance + ", \""
+                                + method.getName()
+                                + "\", $args, $sig, \""
+                                + returnTypeAsString
+                              + "\");"
+                              + "if (value != " + MockGateway.class.getName() + ".PROCEED) " + "return "
+                              + returnValue + "; ";
+
+        method.insertBefore("{ " + code + "}");
+    }
+
+    private boolean isBridgeMethod(CtMethod method) {return (method.getMethodInfo2().getAccessFlags() & AccessFlag.BRIDGE) != 0;}
+
+    private String classOrInstance(CtMethod method) {
+        String classOrInstance = "this";
+        if (Modifier.isStatic(method.getModifiers())) {
+            classOrInstance = "$class";
+        }
+        return classOrInstance;
+    }
+
+    private void modifyNativeMethod(CtMethod method, CtClass returnTypeAsCtClass,
+                                    String returnTypeAsString) throws CannotCompileException {
+        String methodName = method.getName();
+        String returnValue = "($r)value";
+
+        if (returnTypeAsCtClass.equals(CtClass.voidType)) {
+            returnValue = VOID;
+        }
+
+        String classOrInstance = classOrInstance(method);
+        method.setModifiers(method.getModifiers() - Modifier.NATIVE);
+        String code = "Object value = "
+                              + MockGateway.class.getName()
+                              + ".methodCall("
+                                + classOrInstance
+                                + ", \""
+                                + method.getName()
+                                + "\", $args, $sig, \""
+                                + returnTypeAsString
+                              + "\");"
+                              + "if (value != "
+                                + MockGateway.class.getName() + ".PROCEED) "
+                                + "return "
+                                + returnValue + "; "
+                                    + "throw new java.lang.UnsupportedOperationException(\"" + methodName + " is native\");";
+        method.setBody("{" + code + "}");
+    }
+
     private String getReturnTypeAsString(final CtMethod method) throws NotFoundException {
         CtClass returnType = method.getReturnType();
         String returnTypeAsString = VOID;
@@ -219,7 +271,7 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
 
     /**
      * @return The correct return type, i.e. takes care of casting the a wrapper
-     *         type to primitive type if needed.
+     * type to primitive type if needed.
      */
     private String getCorrectReturnValueType(final CtClass returnTypeAsCtClass) {
         final String returnTypeAsString = returnTypeAsCtClass.getName();
@@ -240,17 +292,10 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
         return returnValue;
     }
 
-    public CtClass transform(final CtClass clazz) throws Exception {
-        if (clazz.isFrozen()) {
-            clazz.defrost();
-        }
-        return transformMockClass(clazz);
-
-
+    private boolean isNotSyntheticField(FieldInfo fieldInfo) {
+        return (fieldInfo.getAccessFlags() & AccessFlag.SYNTHETIC) == 0;
     }
 
-    protected abstract CtClass transformMockClass(CtClass clazz) throws CannotCompileException, NotFoundException;
-
     protected final class PowerMockExpressionEditor extends ExprEditor {
         private final CtClass clazz;
 
@@ -262,9 +307,12 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
         public void edit(FieldAccess f) throws CannotCompileException {
             if (f.isReader()) {
                 CtClass returnTypeAsCtClass;
+                FieldInfo fieldInfo;
 
                 try {
-                    returnTypeAsCtClass = f.getField().getType();
+                    CtField field = f.getField();
+                    returnTypeAsCtClass = field.getType();
+                    fieldInfo = field.getFieldInfo2();
                 } catch (NotFoundException e) {
                         /*
                          * If multiple java agents are active (in INST_REDEFINE mode), the types implicitly loaded by javassist from disk
@@ -275,15 +323,21 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
                          */
                     return;
                 }
-                StringBuilder code = new StringBuilder();
-                code.append("{Object value =  ").append(MockGateway.class.getName()).append(".fieldCall(").append("$0,$class,\"").append(
-                        f.getFieldName()).append("\",$type);");
-                code.append("if(value == ").append(MockGateway.class.getName()).append(".PROCEED) {");
-                code.append("	$_ = $proceed($$);");
-                code.append("} else {");
-                code.append("	$_ = ").append(getCorrectReturnValueType(returnTypeAsCtClass)).append(";");
-                code.append("}}");
-                f.replace(code.toString());
+
+                if (isNotSyntheticField(fieldInfo)) {
+                    String code = "{Object value =  " +
+                                          MockGateway.class.getName() +
+                                          ".fieldCall(" +
+                                          "$0,$class,\"" +
+                                          f.getFieldName() +
+                                          "\",$type);" +
+                                          "if(value == " + MockGateway.class.getName() + ".PROCEED) {" +
+                                          "	$_ = $proceed($$);" +
+                                          "} else {" +
+                                          "	$_ = " + getCorrectReturnValueType(returnTypeAsCtClass) + ";" +
+                                          "}}";
+                    f.replace(code);
+                }
             }
         }
 
@@ -292,12 +346,19 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
             try {
                 final CtMethod method = m.getMethod();
                 final CtClass declaringClass = method.getDeclaringClass();
+
                 if (declaringClass != null) {
                     if (shouldTreatAsSystemClassCall(declaringClass)) {
                         StringBuilder code = new StringBuilder();
                         code.append("{Object classOrInstance = null; if($0!=null){classOrInstance = $0;} else { classOrInstance = $class;}");
-                        code.append("Object value =  ").append(MockGateway.class.getName()).append(".methodCall(").append("classOrInstance,\"")
-                                .append(m.getMethodName()).append("\",$args, $sig,\"").append(getReturnTypeAsString(method)).append("\");");
+                        code.append("Object value =  ")
+                            .append(MockGateway.class.getName())
+                            .append(".methodCall(")
+                            .append("classOrInstance,\"")
+                            .append(m.getMethodName())
+                            .append("\",$args, $sig,\"")
+                            .append(getReturnTypeAsString(method))
+                            .append("\");");
                         code.append("if(value == ").append(MockGateway.class.getName()).append(".PROCEED) {");
                         code.append("	$_ = $proceed($$);");
                         code.append("} else {");
@@ -353,7 +414,9 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
                  */
                 addNewDeferConstructor(clazz);
                 final StringBuilder code = new StringBuilder();
-                code.append("{Object value =").append(MockGateway.class.getName()).append(".constructorCall($class, $args, $sig);");
+                code.append("{Object value =")
+                    .append(MockGateway.class.getName())
+                    .append(".constructorCall($class, $args, $sig);");
                 code.append("if (value != ").append(MockGateway.class.getName()).append(".PROCEED){");
 
                 /*
@@ -362,7 +425,7 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
                 if (superclass.getName().equals(Object.class.getName())) {
                     code.append(" super();");
                 } else {
-                    code.append(" super((" + IndicateReloadClass.class.getName() + ") null);");
+                    code.append(" super((").append(IndicateReloadClass.class.getName()).append(") null);");
                 }
                 code.append("} else {");
                 code.append("   $proceed($$);");
@@ -385,7 +448,7 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
                 superClass = clazz.getSuperclass();
             } catch (NotFoundException e1) {
                 throw new IllegalArgumentException("Internal error: Failed to get superclass for " + clazz.getName()
-                        + " when about to create a new default constructor.");
+                                                           + " when about to create a new default constructor.");
             }
 
             ClassPool classPool = clazz.getClassPool();
@@ -399,7 +462,7 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
                 constructorType = classPool.get(IndicateReloadClass.class.getName());
             } catch (NotFoundException e) {
                 throw new IllegalArgumentException("Internal error: failed to get the " + IndicateReloadClass.class.getName()
-                        + " when added defer constructor.");
+                                                           + " when added defer constructor.");
             }
             clazz.defrost();
             if (superClass.getName().equals(Object.class.getName())) {
@@ -420,20 +483,21 @@ public abstract class AbstractMainMockTransformer implements MockTransformer {
 
         @Override
         public void edit(NewExpr e) throws CannotCompileException {
-            final StringBuilder code = new StringBuilder();
-            code.append("Object instance =").append(MockGateway.class.getName()).append(".newInstanceCall($type,$args,$sig);");
-            code.append("if(instance != ").append(MockGateway.class.getName()).append(".PROCEED) {");
-            code.append("	if(instance instanceof java.lang.reflect.Constructor) {");
+            String code = "Object instance =" +
+                                  MockGateway.class.getName() +
+                                  ".newInstanceCall($type,$args,$sig);" +
+                                  "if(instance != " + MockGateway.class.getName() + ".PROCEED) {" +
+                                  "	if(instance instanceof java.lang.reflect.Constructor) {"
+                                  +
+                                  "		$_ = ($r) sun.reflect.ReflectionFactory.getReflectionFactory().newConstructorForSerialization($type, java.lang.Object.class.getDeclaredConstructor(null)).newInstance(null);" +
+                                  "	} else {" +
+                                  "		$_ = ($r) instance;" +
+                                  "	}" +
+                                  "} else {" +
+                                  "	$_ = $proceed($$);" +
+                                  "}";
             // TODO Change to objenisis instead
-            code
-                    .append("		$_ = ($r) sun.reflect.ReflectionFactory.getReflectionFactory().newConstructorForSerialization($type, java.lang.Object.class.getDeclaredConstructor(null)).newInstance(null);");
-            code.append("	} else {");
-            code.append("		$_ = ($r) instance;");
-            code.append("	}");
-            code.append("} else {");
-            code.append("	$_ = $proceed($$);");
-            code.append("}");
-            e.replace(code.toString());
+            e.replace(code);
         }
     }
 }
